require File.expand_path('../../helper', __FILE__)
require 'stow'

describe Rise::Uploadable do
  before do
    @test_dir = File.expand_path('../../', __FILE__)
    @fixture_dir = File.join(@test_dir, 'fixtures')
    @store_dir = File.join(@test_dir, 'tmp/uploads')
    @cache_dir = File.join(@test_dir, 'tmp/cache')
    FileUtils.mkdir_p(@store_dir)
    FileUtils.mkdir_p(@cache_dir)
    @klass = Class.new do
      include Stow::Storable
      include Rise::Uploadable
    end
    @klass.storage :local, base: @store_dir
    @klass.cache = Stow.new(:local, base: @cache_dir)
  end
  
  after do
    # Specifying explicitly to prevent mistakes with rm_rf.
    FileUtils.rm_rf(File.expand_path('../../tmp/uploads', __FILE__))
    FileUtils.rm_rf(File.expand_path('../../tmp/cache', __FILE__))
  end
  
  subject { @klass.new }

  describe :upload= do
    before do
      Timecop.freeze(Time.local(1979))
      Process.stubs(:pid).returns(12345)
      Rise.stubs(:rand).with(9999).returns('0001')
    end
    
    after do
      Timecop.return
    end
    
    it "does nothing for nil" do
      subject.upload = nil
      assert_equal false, subject.cached?
    end
    
    it "does nothing for empty string" do
      subject.upload = ''
      assert_equal false, subject.cached?
    end
    
    it "raises multipart error" do
      assert_raises(Rise::MultipartError) { subject.upload = 'a string' }
    end
    
    it "caches physical file" do
      subject.upload = File.open(File.join(@fixture_dir, 'file.txt'))
      path = File.join(@cache_dir, '19790101-0000-12345-0001/file.txt')
      assert_equal 'Hello World', File.read(path)
    end
    
    it "caches uploaded file" do
      subject.upload = stub(read: 'Hello World', size: 11, original_filename: 'file.txt')
      path = File.join(@cache_dir, '19790101-0000-12345-0001/file.txt')
      assert_equal 'Hello World', File.read(path)
    end
  end
  
  describe :filename do
    it "returns original filename by default" do
      subject.stubs(:original_filename).returns('original.txt')
      assert_equal 'original.txt', subject.filename
    end
  end
  
  describe :original_filename do
    it "returns nil by default" do
      assert_nil subject.original_filename
    end
    
    it "determined from physical file" do
      subject.upload = File.open(File.join(@fixture_dir, 'file.txt'))
      assert_equal 'file.txt', subject.original_filename
    end
    
    it "determined from uploaded file" do
      subject.upload = stub(read: 'content', size: 7, original_filename: 'file.txt')
      assert_equal 'file.txt', subject.original_filename
    end
    
    it "determined from cached file" do
      previous = @klass.new
      previous.upload = stub(read: 'content', size: 7, original_filename: 'file.txt')      
      subject.cache = previous.cache
      assert_equal 'file.txt', subject.original_filename
    end
        
    it "sanitizes internet explorer paths" do
      subject.upload = stub(read: 'content', size: 7, original_filename: 'some\path\to\file.txt')
      assert_equal 'file.txt', subject.original_filename
    end
    
    it "sanitizes based on allowed characters" do
      subject.upload = stub(read: 'content', size: 7, original_filename: 'ab_AB-01+.&$%*.txt')
      assert_equal 'ab_AB-01+.-.txt', subject.original_filename
    end
    
    it "sanitizes dot" do
      subject.upload = stub(read: 'content', size: 7, original_filename: '.')
      assert_equal 'untitled', subject.original_filename
    end
    
    it "sanitizes multiple dots" do
      subject.upload = stub(read: 'content', size: 7, original_filename: '...')
      assert_equal 'untitled', subject.original_filename
    end
    
    it "sanitizes blank" do
      subject.upload = stub(read: 'content', size: 7, original_filename: '')
      assert_equal 'untitled', subject.original_filename
    end
    
    it "sanitizes only dashes" do
      subject.upload = stub(read: 'content', size: 7, original_filename: '---')
      assert_equal 'untitled', subject.original_filename
    end
    
    it "sanitizes only dots and dashes" do
      subject.upload = stub(read: 'content', size: 7, original_filename: '.-..--')
      assert_equal 'untitled', subject.original_filename
    end
    
    it "sanitizes multiple dashes" do
      subject.upload = stub(read: 'content', size: 7, original_filename: 'a---b')
      assert_equal 'a-b', subject.original_filename
    end
    
    it "uses custom sanitizer" do
      Rise.stubs(:sanitize_regexp).returns(/[^a-z0-9-]/)
      subject.upload = stub(read: 'content', size: 7, original_filename: 'ab_AB-01+.&$%*.txt')
      assert_equal 'ab-01-txt', subject.original_filename
    end
  end
  
  describe :cache do
    before do
      Timecop.freeze(Time.local(1979))
      Process.stubs(:pid).returns(12345)
      Rise.stubs(:rand).with(9999).returns('0001')
    end
    
    after do
      Timecop.return
    end
    
    it "returns nil by default" do
      assert_nil subject.cache
    end
    
    it "returns cache path when physical file" do
      subject.upload = File.open(File.join(@fixture_dir, 'file.txt'))
      assert_equal '19790101-0000-12345-0001/file.txt', subject.cache
    end
    
    it "returns cache path when uploaded file" do
      subject.upload = stub(read: 'content', size: 7, original_filename: 'file.txt')
      assert_equal '19790101-0000-12345-0001/file.txt', subject.cache
    end
    
    it "returns cache path after retrieval" do
      previous = @klass.new
      previous.upload = stub(read: 'content', size: 7, original_filename: 'file.txt')      
      subject.cache = previous.cache
      assert_equal '19790101-0000-12345-0001/file.txt', subject.cache
    end
    
    it "returns nil after removal" do
      subject.upload = stub(read: 'Hello World', size: 11, original_filename: 'file.txt')
      subject.store!
      subject.remove!
      assert_nil subject.cache
    end
  end
  
  describe :cache= do
    before do
      Timecop.freeze(Time.local(1979))
      Process.stubs(:pid).returns(12345)
      Rise.stubs(:rand).with(9999).returns('0001')
    end
    
    after do
      Timecop.return
    end

    it "does nothing for nil" do
      subject.cache = nil
      assert_equal nil, subject.cache
    end
    
    it "does nothing for empty string" do
      subject.cache = ''
      assert_equal nil, subject.cache
    end
    
    it "does nothing if file already retrieved" do
      subject.upload = stub(read: 'content', size: 7, original_filename: 'file.txt')  
      id = subject.file.object_id
      subject.cache = '19790101-0000-12345-0001/file.txt'
      assert_equal '19790101-0000-12345-0001/file.txt', subject.cache
      assert_equal id, subject.file.object_id
    end
    
    it "retrieves file when not already retrieved" do
      previous = @klass.new
      previous.upload = stub(read: 'content', size: 7, original_filename: 'file.txt')      
      subject.cache = previous.cache
      assert_equal '19790101-0000-12345-0001/file.txt', subject.file.path
    end
    
    it "raises when invalid cache prefix" do    
      assert_raises(Rise::InvalidCachePrefix) { subject.cache = 'wrong-format' }
    end
  end
  
  describe :cached? do
    it "defaults to false" do
      assert_equal false, subject.cached?
    end
    
    it "returns true after upload" do
      subject.upload = stub(read: 'content', size: 7, original_filename: 'file.txt')  
      assert_equal true, subject.cached?
    end
    
    it "returns true after retrieving from cache" do
      previous = @klass.new
      previous.upload = stub(read: 'content', size: 7, original_filename: 'file.txt')      
      subject.cache = previous.cache
      assert_equal true, subject.cached?
    end
    
    it "returns false after removal" do
      subject.upload = stub(read: 'Hello World', size: 11, original_filename: 'file.txt')
      subject.store!
      subject.remove!
      assert_equal false, subject.cached?
    end
  end
  
  describe :store! do
    it "does nothing when no cache" do
      subject.store!
    end
    
    it "stores uploaded file" do
      subject.upload = stub(read: 'Hello World', size: 11, original_filename: 'file.txt')
      subject.store!
      assert_equal 'Hello World', File.read(File.join(@store_dir, 'file.txt'))
    end
    
    it "deletes when marked for removal with '1'" do
      subject.upload = stub(read: 'Hello World', size: 11, original_filename: 'file.txt')
      subject.store!
      subject.remove = '1'
      subject.store!
      assert_equal false, File.file?(File.join(@store_dir, 'file.txt'))
    end
    
    it "deletes when marked for removal with true" do
      subject.upload = stub(read: 'Hello World', size: 11, original_filename: 'file.txt')
      subject.store!
      subject.remove = true
      subject.store!
      assert_equal false, File.file?(File.join(@store_dir, 'file.txt'))
    end
  end
end




